from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

@dataclass
class Country(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: Dict[str, Any] = field(default_factory=dict)

    # The FlagUrl property
    flag_url: Optional[str] = None
    # The IsEU property
    is_e_u: Optional[bool] = None
    # The IsoCode property
    iso_code: Optional[str] = None
    # The Name property
    name: Optional[str] = None
    # The PopList property
    pop_list: Optional[List[str]] = None
    # The TaxPrefix property
    tax_prefix: Optional[str] = None
    # The TaxRate property
    tax_rate: Optional[int] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> Country:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: Country
        """
        if not parse_node:
            raise TypeError("parse_node cannot be null.")
        return Country()
    
    def get_field_deserializers(self,) -> Dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: Dict[str, Callable[[ParseNode], None]]
        """
        fields: Dict[str, Callable[[Any], None]] = {
            "FlagUrl": lambda n : setattr(self, 'flag_url', n.get_str_value()),
            "IsEU": lambda n : setattr(self, 'is_e_u', n.get_bool_value()),
            "IsoCode": lambda n : setattr(self, 'iso_code', n.get_str_value()),
            "Name": lambda n : setattr(self, 'name', n.get_str_value()),
            "PopList": lambda n : setattr(self, 'pop_list', n.get_collection_of_primitive_values(str)),
            "TaxPrefix": lambda n : setattr(self, 'tax_prefix', n.get_str_value()),
            "TaxRate": lambda n : setattr(self, 'tax_rate', n.get_int_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if not writer:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("FlagUrl", self.flag_url)
        writer.write_bool_value("IsEU", self.is_e_u)
        writer.write_str_value("IsoCode", self.iso_code)
        writer.write_str_value("Name", self.name)
        writer.write_collection_of_primitive_values("PopList", self.pop_list)
        writer.write_str_value("TaxPrefix", self.tax_prefix)
        writer.write_int_value("TaxRate", self.tax_rate)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
from kiota_abstractions.api_client_builder import enable_backing_store_for_serialization_writer_factory, register_default_deserializer, register_default_serializer
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.serialization import ParseNodeFactoryRegistry, SerializationWriterFactoryRegistry
from kiota_serialization_form.form_parse_node_factory import FormParseNodeFactory
from kiota_serialization_form.form_serialization_writer_factory import FormSerializationWriterFactory
from kiota_serialization_json.json_parse_node_factory import JsonParseNodeFactory
from kiota_serialization_json.json_serialization_writer_factory import JsonSerializationWriterFactory
from kiota_serialization_multipart.multipart_serialization_writer_factory import MultipartSerializationWriterFactory
from kiota_serialization_text.text_parse_node_factory import TextParseNodeFactory
from kiota_serialization_text.text_serialization_writer_factory import TextSerializationWriterFactory
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .item.with_storage_zone_name_item_request_builder import WithStorageZoneNameItemRequestBuilder

class EdgeStorageApiClient(BaseRequestBuilder):
    """
    The main entry point of the SDK, exposes the configuration and the fluent API.
    """
    def __init__(self,request_adapter: RequestAdapter) -> None:
        """
        Instantiates a new EdgeStorageApiClient and sets the default values.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        if not request_adapter:
            raise TypeError("request_adapter cannot be null.")
        super().__init__(request_adapter, "{+baseurl}", None)
        register_default_serializer(JsonSerializationWriterFactory)
        register_default_serializer(TextSerializationWriterFactory)
        register_default_serializer(FormSerializationWriterFactory)
        register_default_serializer(MultipartSerializationWriterFactory)
        register_default_deserializer(JsonParseNodeFactory)
        register_default_deserializer(TextParseNodeFactory)
        register_default_deserializer(FormParseNodeFactory)
        if not self.request_adapter.base_url:
            self.request_adapter.base_url = "https://{region}.bunnycdn.com"
        self.path_parameters["base_url"] = self.request_adapter.base_url
    
    def by_storage_zone_name(self,storage_zone_name: str) -> WithStorageZoneNameItemRequestBuilder:
        """
        Gets an item from the EdgeStorageApiClient.item collection
        param storage_zone_name: the name of your storage zone where you are connecting to.
        Returns: WithStorageZoneNameItemRequestBuilder
        """
        if not storage_zone_name:
            raise TypeError("storage_zone_name cannot be null.")
        from .item.with_storage_zone_name_item_request_builder import WithStorageZoneNameItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["storageZoneName"] = storage_zone_name
        return WithStorageZoneNameItemRequestBuilder(self.request_adapter, url_tpl_params)
    




from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .action import Action
    from .trigger import Trigger

@dataclass
class EdgeRule(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: Dict[str, Any] = field(default_factory=dict)

    # The Action parameter 1. The value depends on other parameters of the edge rule.
    action_parameter1: Optional[str] = None
    # The Action parameter 2. The value depends on other parameters of the edge rule.
    action_parameter2: Optional[str] = None
    # The ActionType property
    action_type: Optional[float] = None
    # The description of the edge rule
    description: Optional[str] = None
    # Determines if the edge rule is currently enabled or not
    enabled: Optional[bool] = None
    # The ExtraActions property
    extra_actions: Optional[List[Action]] = None
    # The unique GUID of the edge rule
    guid: Optional[str] = None
    # The TriggerMatchingType property
    trigger_matching_type: Optional[float] = None
    # The Triggers property
    triggers: Optional[List[Trigger]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> EdgeRule:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: EdgeRule
        """
        if not parse_node:
            raise TypeError("parse_node cannot be null.")
        return EdgeRule()
    
    def get_field_deserializers(self,) -> Dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: Dict[str, Callable[[ParseNode], None]]
        """
        from .action import Action
        from .trigger import Trigger

        from .action import Action
        from .trigger import Trigger

        fields: Dict[str, Callable[[Any], None]] = {
            "ActionParameter1": lambda n : setattr(self, 'action_parameter1', n.get_str_value()),
            "ActionParameter2": lambda n : setattr(self, 'action_parameter2', n.get_str_value()),
            "ActionType": lambda n : setattr(self, 'action_type', n.get_float_value()),
            "Description": lambda n : setattr(self, 'description', n.get_str_value()),
            "Enabled": lambda n : setattr(self, 'enabled', n.get_bool_value()),
            "ExtraActions": lambda n : setattr(self, 'extra_actions', n.get_collection_of_object_values(Action)),
            "Guid": lambda n : setattr(self, 'guid', n.get_str_value()),
            "TriggerMatchingType": lambda n : setattr(self, 'trigger_matching_type', n.get_float_value()),
            "Triggers": lambda n : setattr(self, 'triggers', n.get_collection_of_object_values(Trigger)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if not writer:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("ActionParameter1", self.action_parameter1)
        writer.write_str_value("ActionParameter2", self.action_parameter2)
        writer.write_float_value("ActionType", self.action_type)
        writer.write_str_value("Description", self.description)
        writer.write_bool_value("Enabled", self.enabled)
        writer.write_collection_of_object_values("ExtraActions", self.extra_actions)
        writer.write_str_value("Guid", self.guid)
        writer.write_float_value("TriggerMatchingType", self.trigger_matching_type)
        writer.write_collection_of_object_values("Triggers", self.triggers)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from ..models.dns_zone.dns_zone import DnsZone
    from ..models.structured_bad_request_response import StructuredBadRequestResponse
    from .checkavailability.checkavailability_request_builder import CheckavailabilityRequestBuilder
    from .dnszone_get_response import DnszoneGetResponse
    from .dnszone_post_request_body import DnszonePostRequestBody
    from .item.item_request_builder import ItemRequestBuilder

class DnszoneRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /dnszone
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, Dict[str, Any]]) -> None:
        """
        Instantiates a new DnszoneRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/dnszone?page={page}&perPage={perPage}&search={search}", path_parameters)
    
    def by_id(self,id: int) -> ItemRequestBuilder:
        """
        Gets an item from the BunnyApiClient.dnszone.item collection
        param id: The ID of the DNS Zone that will be returned
        Returns: ItemRequestBuilder
        """
        if not id:
            raise TypeError("id cannot be null.")
        from .item.item_request_builder import ItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["%2Did"] = id
        return ItemRequestBuilder(self.request_adapter, url_tpl_params)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[DnszoneRequestBuilderGetQueryParameters]] = None) -> Optional[DnszoneGetResponse]:
        """
        [ListDnsZones API Docs](https://docs.bunny.net/reference/dnszonepublic_index)
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[DnszoneGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .dnszone_get_response import DnszoneGetResponse

        return await self.request_adapter.send_async(request_info, DnszoneGetResponse, None)
    
    async def post(self,body: DnszonePostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[DnsZone]:
        """
        [AddDnsZone API Docs](https://docs.bunny.net/reference/dnszonepublic_add)
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[DnsZone]
        """
        if not body:
            raise TypeError("body cannot be null.")
        request_info = self.to_post_request_information(
            body, request_configuration
        )
        from ..models.structured_bad_request_response import StructuredBadRequestResponse

        error_mapping: Dict[str, ParsableFactory] = {
            "400": StructuredBadRequestResponse,
        }
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from ..models.dns_zone.dns_zone import DnsZone

        return await self.request_adapter.send_async(request_info, DnsZone, error_mapping)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[DnszoneRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        [ListDnsZones API Docs](https://docs.bunny.net/reference/dnszonepublic_index)
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_post_request_information(self,body: DnszonePostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        [AddDnsZone API Docs](https://docs.bunny.net/reference/dnszonepublic_add)
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if not body:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.POST, '{+baseurl}/dnszone', self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    def with_url(self,raw_url: str) -> DnszoneRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: DnszoneRequestBuilder
        """
        if not raw_url:
            raise TypeError("raw_url cannot be null.")
        return DnszoneRequestBuilder(self.request_adapter, raw_url)
    
    @property
    def checkavailability(self) -> CheckavailabilityRequestBuilder:
        """
        The checkavailability property
        """
        from .checkavailability.checkavailability_request_builder import CheckavailabilityRequestBuilder

        return CheckavailabilityRequestBuilder(self.request_adapter, self.path_parameters)
    
    @dataclass
    class DnszoneRequestBuilderGetQueryParameters():
        """
        [ListDnsZones API Docs](https://docs.bunny.net/reference/dnszonepublic_index)
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if not original_name:
                raise TypeError("original_name cannot be null.")
            if original_name == "per_page":
                return "perPage"
            if original_name == "page":
                return "page"
            if original_name == "search":
                return "search"
            return original_name
        
        page: Optional[int] = None

        per_page: Optional[int] = None

        # The search term that will be used to filter the results
        search: Optional[str] = None

    
    @dataclass
    class DnszoneRequestBuilderGetRequestConfiguration(RequestConfiguration[DnszoneRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class DnszoneRequestBuilderPostRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from kiota_abstractions.authentication.api_key_authentication_provider import (
    ApiKeyAuthenticationProvider,
    KeyLocation,
)
from kiota_http.httpx_request_adapter import HttpxRequestAdapter


from .generated.BunnyApiClient.bunny_api_client import BunnyApiClient
from .generated.EdgeStorageApiClient.edge_storage_api_client import EdgeStorageApiClient
from .generated.StreamApiClient.stream_api_client import StreamApiClient


def createBunnyApiClient(api_key: str) -> BunnyApiClient:
    authentication_provider = ApiKeyAuthenticationProvider(
        key_location=KeyLocation.Header, api_key=api_key, parameter_name="AccessKey"
    )

    request_adapter = HttpxRequestAdapter(authentication_provider)

    return BunnyApiClient(request_adapter)


def createEdgeStorageApiClient(api_key: str) -> EdgeStorageApiClient:
    authentication_provider = ApiKeyAuthenticationProvider(
        key_location=KeyLocation.Header, api_key=api_key, parameter_name="AccessKey"
    )

    request_adapter = HttpxRequestAdapter(authentication_provider)

    return EdgeStorageApiClient(request_adapter)


def createStreamApiClient(api_key: str) -> StreamApiClient:
    authentication_provider = ApiKeyAuthenticationProvider(
        key_location=KeyLocation.Header, api_key=api_key, parameter_name="AccessKey"
    )

    request_adapter = HttpxRequestAdapter(authentication_provider)

    return StreamApiClient(request_adapter)



from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

@dataclass
class Hostname(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: Dict[str, Any] = field(default_factory=dict)

    # Contains the Base64Url encoded certificate for the hostname
    certificate: Optional[str] = None
    # Contains the Base64Url encoded certificate key for the hostname
    certificate_key: Optional[str] = None
    # Determines if the Force SSL feature is enabled
    force_s_s_l: Optional[bool] = None
    # Determines if the hostname has an SSL certificate configured
    has_certificate: Optional[bool] = None
    # The unique ID of the hostname
    id: Optional[int] = None
    # Determines if this is a system hostname controlled by bunny.net
    is_system_hostname: Optional[bool] = None
    # The hostname value for the domain name
    value: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> Hostname:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: Hostname
        """
        if not parse_node:
            raise TypeError("parse_node cannot be null.")
        return Hostname()
    
    def get_field_deserializers(self,) -> Dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: Dict[str, Callable[[ParseNode], None]]
        """
        fields: Dict[str, Callable[[Any], None]] = {
            "Certificate": lambda n : setattr(self, 'certificate', n.get_str_value()),
            "CertificateKey": lambda n : setattr(self, 'certificate_key', n.get_str_value()),
            "ForceSSL": lambda n : setattr(self, 'force_s_s_l', n.get_bool_value()),
            "HasCertificate": lambda n : setattr(self, 'has_certificate', n.get_bool_value()),
            "Id": lambda n : setattr(self, 'id', n.get_int_value()),
            "IsSystemHostname": lambda n : setattr(self, 'is_system_hostname', n.get_bool_value()),
            "Value": lambda n : setattr(self, 'value', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if not writer:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("Certificate", self.certificate)
        writer.write_str_value("CertificateKey", self.certificate_key)
        writer.write_bool_value("ForceSSL", self.force_s_s_l)
        writer.write_bool_value("HasCertificate", self.has_certificate)
        writer.write_int_value("Id", self.id)
        writer.write_bool_value("IsSystemHostname", self.is_system_hostname)
        writer.write_str_value("Value", self.value)
        writer.write_additional_data_value(self.additional_data)
    




from __future__ import annotations
from kiota_abstractions.api_client_builder import enable_backing_store_for_serialization_writer_factory, register_default_deserializer, register_default_serializer
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.serialization import ParseNodeFactoryRegistry, SerializationWriterFactoryRegistry
from kiota_serialization_form.form_parse_node_factory import FormParseNodeFactory
from kiota_serialization_form.form_serialization_writer_factory import FormSerializationWriterFactory
from kiota_serialization_json.json_parse_node_factory import JsonParseNodeFactory
from kiota_serialization_json.json_serialization_writer_factory import JsonSerializationWriterFactory
from kiota_serialization_multipart.multipart_serialization_writer_factory import MultipartSerializationWriterFactory
from kiota_serialization_text.text_parse_node_factory import TextParseNodeFactory
from kiota_serialization_text.text_serialization_writer_factory import TextSerializationWriterFactory
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .library.library_request_builder import LibraryRequestBuilder
    from .o_embed.o_embed_request_builder import OEmbedRequestBuilder

class StreamApiClient(BaseRequestBuilder):
    """
    The main entry point of the SDK, exposes the configuration and the fluent API.
    """
    def __init__(self,request_adapter: RequestAdapter) -> None:
        """
        Instantiates a new StreamApiClient and sets the default values.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        if not request_adapter:
            raise TypeError("request_adapter cannot be null.")
        super().__init__(request_adapter, "{+baseurl}", None)
        register_default_serializer(JsonSerializationWriterFactory)
        register_default_serializer(TextSerializationWriterFactory)
        register_default_serializer(FormSerializationWriterFactory)
        register_default_serializer(MultipartSerializationWriterFactory)
        register_default_deserializer(JsonParseNodeFactory)
        register_default_deserializer(TextParseNodeFactory)
        register_default_deserializer(FormParseNodeFactory)
        if not self.request_adapter.base_url:
            self.request_adapter.base_url = "https://video.bunnycdn.com"
        self.path_parameters["base_url"] = self.request_adapter.base_url
    
    @property
    def library(self) -> LibraryRequestBuilder:
        """
        The library property
        """
        from .library.library_request_builder import LibraryRequestBuilder

        return LibraryRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def o_embed(self) -> OEmbedRequestBuilder:
        """
        The OEmbed property
        """
        from .o_embed.o_embed_request_builder import OEmbedRequestBuilder

        return OEmbedRequestBuilder(self.request_adapter, self.path_parameters)
    

